/*
 * LET BASIC = BASIC + 1
 * Copyright (C) 2006-2008 Rebecca G. Bettencourt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "btype.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

static b_exception be_last_exception = be_none;
static int         be_last_line      = 0;
static char *      be_last_file      = 0;
static int be_close_req = 0;

void be_throw(b_exception x)
{
	be_last_exception = x;
	be_last_line = 0;
	if (be_last_file) {
		free(be_last_file);
		be_last_file = 0;
	}
}

void be_cthrow(b_exception x)
{
	if (be_last_exception == be_none) be_throw(x);
}

void be_set_lineNum(int i)
{
	be_last_line = i;
}

void be_set_fileName(char * f)
{
	if (be_last_file) free(be_last_file);
	be_last_file = strdup(f);
}

int be_checkerr()
{
	return (be_last_exception != be_none);
}

b_exception be_lasterr()
{
	return be_last_exception;
}

int be_lastline()
{
	return be_last_line;
}

char * be_lastfile()
{
	return be_last_file;
}

void be_clearerr()
{
	be_last_exception = be_none;
	be_last_line = 0;
	if (be_last_file) {
		free(be_last_file);
		be_last_file = 0;
	}
}

char * be_errstr(b_exception x)
{
	switch (x) {
	case be_syntax:		return "?SYNTAX ERROR";
	case be_subscript:	return "?SUBSCRIPT OUT OF RANGE";
	case be_type:		return "?INVALID TYPE";
	case be_div0:		return "?DIVISION BY ZERO";
	case be_formula:	return "?FORMULA TOO COMPLEX";
	case be_range:		return "?OUT OF RANGE ERROR";
	case be_data:		return "?OUT OF DATA ERROR";
	case be_cont:		return "?CAN'T CONTINUE";
	case be_label:		return "?LABEL NOT FOUND";
	
	case be_noend:		return "?STRUCTURE WITHOUT END";
	case be_else:		return "?ELSE WITHOUT IF";
	case be_endif:		return "?ENDIF WITHOUT IF";
	case be_case:		return "?CASE WITHOUT SWITCH";
	case be_wend:		return "?WEND WITHOUT WHILE";
	case be_loop:		return "?LOOP WITHOUT DO";
	case be_next:		return "?NEXT WITHOUT FOR";
	case be_return:		return "?RETURN WITHOUT GOSUB";
	case be_onerr:		return "?CATCH WITHOUT TRY";
	case be_endtry:		return "?ENDTRY WITHOUT TRY";
	
	case be_memory:		return "?OUT OF MEMORY ERROR";
	case be_nullptr:	return "?NULL OBJECT ERROR";
	case be_undeclared:	return "?UNDECLARED VARIABLE";
	case be_constant:	return "?CANNOT CHANGE CONSTANT";
	case be_arrtype:	return "?CANNOT CHANGE ARRAY TYPE";
	case be_cast:		return "?INCOMPATIBLE VARIABLE TYPES";
	case be_private:	return "?PRIVATE MEMBER ACCESSED OUTSIDE CLASS";
	
	case be_404:		return "?FILE NOT FOUND";
	case be_eof:		return "?END OF FILE";
	case be_xdev:		return "?CROSS FILESYSTEM ERROR";
	case be_perm:		return "?PERMISSION DENIED";
	case be_access:		return "?ACCESS DENIED";
	case be_toolong:	return "?PATH TOO LONG";
	case be_notdir:		return "?NOT A DIRECTORY";
	case be_notfile:	return "?NOT A FILE";
	case be_notempty:	return "?DIRECTORY NOT EMPTY";
	case be_rofs:		return "?READ ONLY FILESYSTEM";
	case be_exists:		return "?FILE ALREADY EXISTS";
	case be_mfile:		return "?MAX FILES OPENED";
	case be_mlink:		return "?MAX LINKS CREATED";
	case be_cloop:		return "?CIRCULAR LINK";
	case be_nospc:		return "?DISK FULL";
	case be_filedes:	return "?BAD FILE DESCRIPTOR";
	case be_inuse:		return "?FILE IN USE";
	case be_noexec:		return "?NOT AN EXECUTABLE FILE";
	case be_toobig:		return "?ARGUMENT LIST TOO LONG";
	case be_io:			return "?I/O ERROR";
	
	case be_break:		return "?BREAK";
	
	case be_max:        return "?UNKNOWN ERROR";
	case be_none:		return "";
	}
	return "";
}

b_exception be_fileerr()
{
	switch (errno) {
	case E2BIG:
		return be_toobig; break;
	case EACCES:
		return be_access; break;
	case EBUSY:
		return be_inuse; break;
	case EBADF:
		return be_filedes; break;
	case EEXIST:
		return be_exists; break;
	case EFAULT:
		return be_perm; break;
#ifndef NO_EFTYPE
	case EFTYPE:
		return be_notdir; break;
#endif
	case EINVAL:
		return be_range; break;
	case EIO:
		return be_io; break;
	case EISDIR:
		return be_notfile; break;
	case ELOOP:
		return be_cloop; break;
	case EMFILE:
		return be_mfile; break;
	case EMLINK:
		return be_mlink; break;
	case ENAMETOOLONG:
		return be_toolong; break;
	case ENOENT:
		return be_404; break;
	case ENOEXEC:
		return be_noexec; break;
	case ENOMEM:
		return be_memory; break;
	case ENOSPC:
		return be_nospc; break;
	case ENOTDIR:
		return be_notdir; break;
	case ENOTEMPTY:
		return be_notempty; break;
	case EPERM:
		return be_perm; break;
	case ERANGE:
		return be_range; break;
	case EROFS:
		return be_rofs; break;
	case EXDEV:
		return be_xdev; break;
	default:
		return be_io; break;
	}
}

void be_sprinterr(char * f)
{
	if (be_last_exception != be_none) {
		sprintf(f, "%s", be_errstr(be_last_exception));
		if (be_last_line) {
			sprintf(f, " ON LINE %d", be_last_line);
		}
		if (be_last_file) {
			sprintf(f, " OF FILE %s", be_last_file);
		}
		sprintf(f, "\n");
	}
}

void be_fprinterr(FILE * f)
{
	if (be_last_exception != be_none) {
		fprintf(f, "%s", be_errstr(be_last_exception));
		if (be_last_line) {
			fprintf(f, " ON LINE %d", be_last_line);
		}
		if (be_last_file) {
			fprintf(f, " OF FILE %s", be_last_file);
		}
		fprintf(f, "\n");
	}
}

void be_printerr()
{
	be_fprinterr(stdout);
}

void be_close()
{
	be_close_req = 1;
}

int be_closed()
{
	return be_close_req;
}

void be_reset()
{
	be_close_req = 0;
	be_clearerr();
}

void be_init()
{
	be_close_req = 0;
	be_clearerr();
}

